#ifndef TRAW_MESH_SURFACE_H
#define TRAW_MESH_SURFACE_H

//
//
// 
#include "../GraphicsAPI/GraphicsAPI.h"

class Td3dDrawable;
class TrawMeshSurface {
	protected:
		static const int kInvalidVertexIndex;

		Td3dDrawable	*fDrawable;
							// for which drawable this MeshSurface is created for

		//
		//  D3DXMESH: in our case:
		//
		//  	1. GetVertexBuffer() returns a pointer to TmyVertex informaiton
		//  	2. GetIndexBuffer() returns a pointer to seqeuence of unsigned short
		//  						3 in a group defining each face of all the triangles
		//  						defeind.
		LPD3DXMESH		fMesh;

		//
		// NumControlVertices is num on each size, e.g. input n means we have a nxn mesh
		//
		// NumVertices = NumControlVertices * NumControlVertices
		// NumFaces = (NumControlVertices - 1)^2 * 2	// (numControlVertices-1) squared then multiply 2
		//
		int    	fNumControlVertices;
		int		fNumFaces;
		int		fNumVertices;

		int			fSelectedVertexIndex;
		D3DXVECTOR3	fSelectedVertex;		// this is only valid when SelectedVertex is valid
											// when valid, this value is copied to the buffer each time we draw


		float  fVertexDistance;

		float ControlVertexSize() const {
						return VertexDistance() * 0.2;
					}


		int  IndexFromRowColumn(int r, int c) const {
						return r * fNumControlVertices + c;
		}

		void LoadSelectedVertexFromD3D();
		void SetSelectedVertexToD3D();

		//
		// Support the drawing of control vertices as boxes
		// 
		// void DrawGridOnLine(int, int, int, int*) const;
		// void DrawGridAt(int, int) const;
		// void DrawControlGrid() const;

	public:


		int 	Resolution() const { return fNumControlVertices; }
		float 	VertexDistance() const { return fVertexDistance; }

		void   	SetToNoSelectedVertex();
		void   	SelectVertex(int r, int c);
		void 	SelectedVertexPosition(D3DXVECTOR3 &p) const;
		void	SetSelectedVertexPosition(const D3DXVECTOR3 &p);

		void 	DrawMeshSurface(Td3dDrawable &d);
		void	DrawMeshControlGrid(Td3dDrawable &d);
		void	DrawNormalVectors(Td3dDrawable &d);

		TrawMeshSurface(LPDIRECT3DDEVICE9, int res=20);
		~TrawMeshSurface();
};
#endif
