#include "T2dBound.h"


T2dBound::T2dBound() {
	// this is no a valid bound
	ResetBound();
}

T2dBound::T2dBound(float xMin, float xMax, float yMin, float yMax) {
	SetBound(xMin, xMax, yMin, yMax);
}

const D3DXVECTOR3& T2dBound::MinPosition() const { return fMin;}
const D3DXVECTOR3& T2dBound::MaxPosition() const { return fMax;}

float T2dBound::Width() const { return fMax.x - fMin.x; }
float T2dBound::Height() const { return fMax.y - fMin.y; }

void T2dBound::SetBound(float xMin, float xMax, float yMin, float yMax) {
	fMin.x = xMin;
	fMax.x = xMax;
	fMin.y = yMin;
	fMax.y = yMax;
}

//
// give a new width and height without
// moving the bound ...
void T2dBound::SetBoundSizeTo(float w, float h) {
	fMax.x = fMin.x + w;
	fMax.y = fMin.y + h;
}

	//
	// Move the entire bound by (dx,dy)
void T2dBound::MoveBoundBy(float dx, float dy) {
	fMin.x += dx;
	fMax.x += dx;
	fMin.y += dy;
	fMax.y += dy;
}

//
// Move the min pt to (x,y) maintining w/h
// of the bound
void T2dBound::SetBoundMinTo(float x, float y) {
	float w = Width();
	float h = Height();
	fMin.x = x;
	fMax.x = x + w;
	fMin.y = y;
	fMax.y = y + h;
}

void T2dBound::ResetBound() {
	// this is no a valid bound
	fMin.x = fMin.y =  100000000;
	fMax.x = fMax.y = -100000000;
}

bool T2dBound::ValidBound() const {
	return (fMax.x > fMin.x) && (fMax.y > fMin.y);
}

void T2dBound::UpdateBound(const D3DXVECTOR3 pt) {
	UpdateRange(pt, x);
	UpdateRange(pt, y);
}

void T2dBound::UpdateBound(const T2dBound &b) {
	UpdateBound(b.fMin);
	UpdateBound(b.fMax);
}

bool T2dBound::BoundIntersect(const T2dBound &b) const {
	return (
			  PtInRange(b.fMin, (*this), x) || 
	          PtInRange(b.fMax, (*this), x) ||
			  PtInRange(fMin, b, x) ||
			  PtInRange(fMax, b, x)
		   ) && 
		   (
			  PtInRange(b.fMin, (*this), y) || 
	          PtInRange(b.fMax, (*this), y) ||
			  PtInRange(fMin, b, y) ||
			  PtInRange(fMax, b, y)
		   );
}

bool T2dBound::PointInBound(const D3DXVECTOR3 pt) const {
	return ( PtInRange(pt, (*this), x) && PtInRange(pt, (*this), y) );
}

void T2dBound::GetBoundaryPoints(D3DXVECTOR3 pt[4]) const {
	pt[0].x = fMin.x;
	pt[0].y = fMax.y;

	pt[1].x = fMax.x;
	pt[1].y = fMax.y;

	pt[2].x = fMax.x;
	pt[2].y = fMin.y;

	pt[3].x = fMin.x;
	pt[3].y = fMin.y;
}
