#include "Tcamera.h"
#include "TaxisFrame.h"
#include "Tline.h"
#include "Tmesh.h"


Tcamera::Tcamera() :
	fEyePosition(0, 0, 0),
	fLookAt(0, 0, 0),
	fUpVector(0, 0, 0)
{
	fFieldOfView = 0.0;
	fAspectRatio = 0.0;
	fNear = 0.0;
	fFar = 0.0;

	fMoveEyePosition = true;
	fShowRailGrid = false;
}


Tcamera::Tcamera(
	D3DXVECTOR3 &eye,
	D3DXVECTOR3 &at,
	D3DXVECTOR3 &up,
	float	fov,
	float 	aspect,
	float	n,
	float	f
	)
{
	fFieldOfView = fov;
	fAspectRatio = aspect;
	fNear = n;
	fFar = f;

	fEyePosition = eye;
	fLookAt = at;
	fUpVector = up;

	fMoveEyePosition = true;
	InitializeUVW();
}



ImplementCameraAttr(FieldOfView, float, float)
ImplementCameraAttr(AspectRatio, float, float)
ImplementCameraAttr(Near,		float, float)
ImplementCameraAttr(Far,			float, float)

ImplementViewAttr(EyePosition,	D3DXVECTOR3, const D3DXVECTOR3&)
ImplementViewAttr(LookAt,		D3DXVECTOR3, const D3DXVECTOR3&)
ImplementViewAttr(UpVector,		D3DXVECTOR3, const D3DXVECTOR3&)

ImplementBoolObjAttr(Tcamera, MoveEyePosition)
ImplementBoolObjAttr(Tcamera, ShowRailGrid)

void 
Tcamera::DefineViewAndProjection(
  	LPDIRECT3DDEVICE9 device
) 
{
  	D3DXMATRIX viewM, projM;

  	D3DXMatrixLookAtRH(&viewM,
  					&fEyePosition,
					&fLookAt,
					&fUpVector);
  	device->SetTransform(D3DTS_VIEW, &viewM);

  	D3DXMatrixPerspectiveFovRH(&projM, 
				  fFieldOfView,
  			      fAspectRatio,  	// aspect ratio 
    			  fNear,		 	// Z near 
				  fFar			 	// Z far
				);
  	device->SetTransform(D3DTS_PROJECTION, &projM);
}
