#include "TgeneralRotation.h"

TgeneralRotation::TgeneralRotation() {
		ResetRotation();
}

void TgeneralRotation::ResetRotation() 
{
		fRotation.x = fRotation.y = fRotation.z = 0;
		D3DXMatrixIdentity(&fRotationMatrix);
}

float TgeneralRotation::x() const { return fRotation.x; }
float TgeneralRotation::y() const { return fRotation.y; }
float TgeneralRotation::z() const { return fRotation.z; }
const D3DXVECTOR3& TgeneralRotation::Rotation() const { return fRotation; }

void TgeneralRotation::UpdateRotationX(float dx) 
{	// new amout of x rotation
		D3DXMATRIX m;
		D3DXMatrixRotationX(&m, dx);
		D3DXMatrixMultiply(&fRotationMatrix, &m, &fRotationMatrix);
		fRotation.x += dx;
}

void TgeneralRotation::UpdateRotationY(float dy) 
{	// new amout of y rotation
		D3DXMATRIX m;
		D3DXMatrixRotationY(&m, dy);
		D3DXMatrixMultiply(&fRotationMatrix, &m, &fRotationMatrix);
		fRotation.y += dy;
}

void TgeneralRotation::UpdateRotationZ(float dz) 
{	// new amout of x rotation
		D3DXMATRIX m;
		D3DXMatrixRotationZ(&m, dz);
		D3DXMatrixMultiply(&fRotationMatrix, &m, &fRotationMatrix);
		fRotation.z += dz;
}

void TgeneralRotation::UpdateRotation(
	const D3DXVECTOR3 &r
) 
	// r is the new combine orientation
	// we must figure out what is the new delta
	// rotation ...
	// 
	// now, we can only work with deltas of one axis at a time
	// (does not make sense to rotate wrt more than one axises)
	// we arbitrary choose testing x, then y, then z, if all three
	// input parameters are non zero, only x will be used.
	//
{
	D3DXVECTOR3 dr = r - fRotation;
	if (dr.x != 0)
		UpdateRotationX(dr.x);
	else if (dr.y != 0)
			UpdateRotationY(dr.y);
		else
			UpdateRotationZ(dr.z);
}

void TgeneralRotation::SetRotation(float x) 
{	
		ResetRotation();
		UpdateRotationX(x);
		UpdateRotationY(x);
		UpdateRotationZ(x);
}

void TgeneralRotation::SetRotation(
		const D3DXVECTOR3 &d
)
{	
		ResetRotation();
		UpdateRotationX(d.x);
		UpdateRotationY(d.y);
		UpdateRotationZ(d.z);
}

void TgeneralRotation::SetOrientation(CONST D3DXMATRIX &m)
{
	fRotationMatrix = m;
}

void TgeneralRotation::SetToStack(
	ID3DXMatrixStack *stack
) const
{
	stack->MultMatrixLocal(&fRotationMatrix);
}
