#ifndef _TXFORMINFO
#define _TXFORMINFO

#include "../StdAfx.h"
#include "GraphicsAPI.h"
#include "TgeneralRotation.h"


class TxformInfo {
	private:
		D3DXVECTOR3 fMove;		
		D3DXVECTOR3 fScale;	
		TgeneralRotation fRotation;  // in radian ...

	public:
		TxformInfo();

		void ResetTransformation();

		void SetUpD3DXformStack(ID3DXMatrixStack *stack) const;

		float MoveX() const;
		float MoveY() const;
		float MoveZ() const;
		const D3DXVECTOR3& Movement() const;

		float ScaleX() const;
		float ScaleY() const;
		float ScaleZ() const;
		const D3DXVECTOR3& Scale() const;

		float RotateX() const;
		float RotateY() const;
		float RotateZ() const;
		const D3DXVECTOR3& Rotation() const;

		void SetMovement(const D3DXVECTOR3 &d);
		void SetMovement(float x, float y, float z);
		void UpdateMovement(float x, float y, float z);

		void SetScale(const D3DXVECTOR3 &s);
		void SetScale(float s);
		void SetScale(float x, float y, float z);
		void UpdateScale(float s);
		void UpdateScale(float x, float y, float z);

		void SetRotation(float theta);
		void SetRotation(float x, float y, float z);
		void UpdateRotation(float x, float y, float z);
		void SetOrientation(const D3DXMATRIX &m);
		const TgeneralRotation& GetRotation() const { return fRotation; }
		const D3DXMATRIX& GetRotationMatrix() const;
};


#endif
