#include "../resource.h"
#include "../Td3dDrawable.h"
#include "../TmodelControl.h"
#include "TmyWorld.h"

#include "../GraphicsAPI/Tcircle.h"
#include "../GraphicsAPI/Tpoint.h"
#include "../GraphicsAPI/Tline.h"



//
// For debugging picking (set by lowerest geometry
// We will draw in the main routine ...
//
#ifdef PICK_RAW_MESH_DEBUG
	#include "../Collections/TrawMesh.h"
	Tpoint sfWorldSph;
	Tline  sfWorldLine;
	TrawMesh *sfPickedMesh = NULL;
#endif

void
TmyWorld::DrawWorld(
	Td3dDrawable *d
)
{
	#ifdef DEBUG_RELEASE
		d->fMeshLibrary.PrintLibraryToLog("My World Draw");
	#endif

	fControl->ControlSetObject();

	// Draw the main fame before the model rotation
	//
	
	
	ID3DXMatrixStack *stack = d->Obj2WldStack();
	LPDIRECT3DDEVICE9 device = d->D3DDevice();

	stack->Push();
		//
		// switch on the lights ... only when are visible ...
		//


	
		if (fPointLgt.VisibilityIsEnabled())
			fPointLgt.SwitchOnLight(device);
		else
			fPointLgt.SwitchOffLight(device);

			
		if (fDirLgt.VisibilityIsEnabled())
			fDirLgt.SwitchOnLight(device);
		else
			fDirLgt.SwitchOffLight(device);

		if (fControl->m_WorldAxisFrame)
			fMainFrame.DrawObject(*d);

	

		fToyCraft.DrawObject(*d);

		
		// now draw the ground ...
		fGround.DrawObject(*d);

		fTeapot.DrawObject(*d);

			
			// now draw the lights themselves ...
			fPointLgt.DrawObject(*d);
			fDirLgt.DrawObject(*d);
			fSpotLgt.DrawObject(*d);

		// 1. Make sure no other lights are on
		//     => Ensuring blocker is pitch-black
		//
		fPointLgt.SwitchOffLight(device);
		fDirLgt.SwitchOffLight(device);
		if (fSpotLgt.VisibilityIsEnabled())
			fSpotLgt.SwitchOnLight(device);
		else
			fSpotLgt.SwitchOffLight(device);
				// now draw the x-ray effect ...
				//
				// 1. switch on blending
				device->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
				// 2. define what we want to do ...
				//
				//   OurColor * (AlphaFromMyColor) + (FrameBufferColor * invSrcColor )
				//
				device->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_INVSRCCOLOR);
				device->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_SRCCOLOR);
					fBlocker.DrawObject(*d);
					// Since the blocker is pitch dark without the spot light
					// the color is zero, thus, we see the black piece of blocker
					//
					// with spot light on, the region lighted by the spotlight
					// has some color, this color value is used as alpah for blending
					// and thus make the blocker appeared to be shine-through by the
					// spot light.
				device->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);
		fSpotLgt.SwitchOffLight(device);
			
	stack->Pop();
	device->SetTransform(D3DTS_WORLD, stack->GetTop());
	
	if (fValidPickRay) {
		Tpoint sph;
		sph.SetPointSize(6);
		sph.DisableLighting();
		Tmaterial &smt = sph.GetMaterial();
		smt.SetEmissive(1, 1, 1);
		sph.SetLocationTo(fPickRayO.x, fPickRayO.y, fPickRayO.z);
		sph.DrawObject(*d);

		D3DXVECTOR3 p2 = fPickRayO + 100 * fPickRayD;
		Tline l;
		l.DisableLighting();
		Tmaterial &lmt = l.GetMaterial();
		lmt.SetEmissive(1, 1, 1);
		l.SetFirstVertex(fPickRayO.x, fPickRayO.y, fPickRayO.z);
		l.SetSecondVertex(p2.x, p2.y, p2.z);
		l.DrawObject(*d);
	}
	#ifdef PICK_RAW_MESH_DEBUG
	if (sfPickedMesh != NULL) {
		sfWorldSph.DrawObject(*d);
		sfWorldLine.DrawObject(*d);
		sfPickedMesh->DrawRawMesh(true);
	}
	#endif
}

