//
// Td3dDrawable_Init.cpp : Contains only the D3D initialization routines
//

#include "stdafx.h"
#include "Td3dDrawable.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//
//***************************************************************
// a *GLOBAL*  variables, back door for echoing error message
// via MessageBox() macro calls. Someone must know a better
// way than this to communicate to the user!!
//***************************************************************
//
Td3dDrawable *gErrorHandler = NULL;	
	//***********************************************************
	// this is really _REALLY_ ugly, but what else can one do?
	//

//
// The followings are from our GraphicsAPI ...
//
#include "./GraphicsAPI/TmyVertex.h"

BOOL Td3dDrawable::InitD3D(
	LPDIRECT3D9		 pD3d
)
{
	gErrorHandler = this;

    // Get the current desktop display mode
    D3DDISPLAYMODE d3ddm;
    if( FAILED( pD3d->GetAdapterDisplayMode( D3DADAPTER_DEFAULT, &d3ddm ) ) ) {
        MessageBox("D3d Creation Failed (2)!");
		return FALSE;
	}

    // Set up the structure used to create the D3dDevice. Most parameters are
    // zeroed out. We set Windowed to TRUE, since we want to do D3d in a
    // window, and then set the SwapEffect to "discard", which is the most
    // efficient method of presenting the back buffer to the display.  And 
    // we request a back buffer format that matches the current desktop display 
    // format.
    D3DPRESENT_PARAMETERS d3dpp; 
		// 1. Init the entire D3DPRESENT structure to 0
    	ZeroMemory( &d3dpp, sizeof(d3dpp) );
		
		// 2. We are creating for one window
    	d3dpp.Windowed = TRUE;

		// 3. We will discard the buffer content after being swapped back (most efficient)
    	d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;
    	d3dpp.BackBufferFormat = d3ddm.Format;

		// 4. Enable Z-buffer stenciling to support 3D ..
 		d3dpp.EnableAutoDepthStencil = TRUE;
 		d3dpp.AutoDepthStencilFormat = D3DFMT_D16;

    // Create the Direct3D device. Here we are using the default adapter (most
    // systems only have one, unless they have multiple graphics hardware cards
    // installed) and requesting the HAL (which is saying we want the hardware
    // device rather than a software one). Software vertex processing is 
    // specified since we know it will work on all cards. On cards that support 
    // hardware vertex processing, though, we would see a big performance gain 
    // by specifying hardware vertex processing.
    if( FAILED( pD3d->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, m_hWnd,
                                      D3DCREATE_SOFTWARE_VERTEXPROCESSING,
                                      &d3dpp, &fD3dDevice ) ) )
    {
        MessageBox("D3d Creation Failed (3)!");
		return FALSE;
    }

	
	// Setup render states
	fD3dDevice->SetRenderState( D3DRS_LIGHTING,	TRUE);  
	fD3dDevice->SetRenderState( D3DRS_SPECULARENABLE, TRUE);
	fD3dDevice->SetRenderState( D3DRS_NORMALIZENORMALS, TRUE);
	fD3dDevice->SetRenderState( D3DRS_AMBIENT, 0x00101010);
								// Ambient light, lights everything in our environment to 
								// 16/255 grey.
	
	fD3dDevice->SetRenderState(D3DRS_EMISSIVEMATERIALSOURCE, D3DMCS_MATERIAL);
	fD3dDevice->SetRenderState(D3DRS_AMBIENTMATERIALSOURCE, D3DMCS_MATERIAL);
	fD3dDevice->SetRenderState(D3DRS_SPECULARMATERIALSOURCE, D3DMCS_MATERIAL);
	fD3dDevice->SetRenderState(D3DRS_DIFFUSEMATERIALSOURCE, D3DMCS_MATERIAL);
								//
								// Other options are: D3DMCS_COLOR1 or COLOR2 as in the channels
								//      from the texture stages. In particular, Diffuse is of particular
								//      interests, when texture mapping is on, we want part of the 
								//      diffuse material coming from the texture, and the other part coming
								//		from the material of the object.
								//
								// all Emissive/Ambient/Diffuse/Specular colors
								// used in the lighting computation will come from
								// materials
	fD3dDevice->SetRenderState(D3DRS_COLORVERTEX, FALSE);
				

	fD3dDevice->SetRenderState( D3DRS_LOCALVIEWER,  TRUE);
	fD3dDevice->SetRenderState( D3DRS_SHADEMODE,	D3DSHADE_GOURAUD);
								// this says linearly interpolate color from vertex to vertex
								// instead of D3DSHADE_GOURAUD, later we will ...

	fD3dDevice->SetRenderState( D3DRS_ZENABLE,	D3DZB_TRUE);
								// Now we are working in 3D, switch on z-buffer
								//     *** NOTE: this is only useful if we specified EnableAutoDepthStencil
								//				  when creating the D3DDevice

	// fD3dDevice->SetRenderState( D3DRS_CULLMODE,	D3DCULL_CW );		
	fD3dDevice->SetRenderState( D3DRS_CULLMODE,	D3DCULL_NONE);		

								// Here Says we want to cull all polys with vertices specified in
								// the clockwise order.
								//		Other two options are: D3DCULL_CCW  or D3DCULL_NONE
								//				where CCW stands for Counter-ClockWise order
								//
						
	//
	// Now create our matrix stack to help us transform from 
	// object space to world space
	if( FAILED( D3DXCreateMatrixStack(0 /* this flag is unsed as of DX9.0 */,
										&fObj2WorldStack) ) )
    {
        MessageBox("D3d Matrix Stack Creation Failed!");
		return FALSE;
    }
	
	if ( FAILED( fObj2WorldStack->LoadIdentity() ) ) 
	{
		MessageBox("D3d Matrix Stack Init to Identity failed!");
		return FALSE;

	}
	#ifdef DEBUG_RELEASE
		CString msg;
		msg.Format("Stack=(%x %x) device(%x %x) World(%x %x) drawable(%x)", 
				fObj2WorldStack, &fObj2WorldStack,
				fD3dDevice, &fD3dDevice,
				fMyWorld, &fMyWorld,
				*this);
		WarningBox(msg);
	#endif

	///////////////////////////////////////////////////////////////////
	//
	// Here we are initializing the Mesh and Texture libries.
	fMeshLibrary.InitializeMeshLibrary(fD3dDevice, fObj2WorldStack);
	fTextureLibrary.InitializeTextureLibrary(fD3dDevice);
	//
	// Create one instacnce of TrawSurfaceMesh for now ...
	//
	fMeshSurfaceLibrary.SetDevice(fD3dDevice);
	fMeshSurfaceLibrary.CreateMeshSurface(0, 20);

	//
	// These two are Win32 function calls (from CWnd)
	//
	ShowWindow(SW_SHOWDEFAULT);
	UpdateWindow();

	//
	// Now let's make sure we know what is our size
	CRect rect;
	this->GetWindowRect(&rect);
	
	fWidth = rect.Width();
	fHeight = rect.Height();


	#ifdef DEBUG_RELEASE
		fMeshLibrary.PrintLibraryToLog("In DrawableInit");
	#endif

    return TRUE;
}
