// TmodelControl.cpp : implementation file
//

#include "StdAfx.h"
#include "ClassExample.h"
#include "TmodelControl.h"
#include "Model/TmyWorld.h"

#include "./GraphicsAPI/TgraphicsObject.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

/////////////////////////////////////////////////////////////////////////////
// TmodelControl dialog


TmodelControl::TmodelControl(CWnd* pParent /*=NULL*/)
	: CDialog(TmodelControl::IDD, pParent)
	, m_Lighting(FALSE)
	, m_GouraudShading(FALSE)
	, m_WireFrame(FALSE)
	, m_ShowNormal(FALSE)
	, m_ShowTexture(FALSE)
	, m_Transparent(FALSE)
	, m_ControlGrid(FALSE)
	, m_ASliderEcho(_T(""))
{
	//{{AFX_DATA_INIT(TmodelControl)
	m_XSliderEcho = _T("");
	m_YSliderEcho = _T("");
	m_ZSliderEcho = _T("");
	m_AxisFrame = FALSE;
	m_Invisible = FALSE;
	m_WorldAxisFrame = FALSE;
	//}}AFX_DATA_INIT

	fWorld = NULL;
	fCurrentObject = NULL;
}


void TmodelControl::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(TmodelControl)
	DDX_Control(pDX, IDC_Y_SLIDER, m_YSlider);
	DDX_Control(pDX, IDC_Z_SLIDER, m_ZSlider);
	DDX_Control(pDX, IDC_X_SLIDER, m_XSlider);
	DDX_Text(pDX, IDC_X_ECHO, m_XSliderEcho);
	DDX_Text(pDX, IDC_Y_ECHO, m_YSliderEcho);
	DDX_Text(pDX, IDC_Z_ECHO, m_ZSliderEcho);
	DDX_Check(pDX, IDC_AXIS_FRAME, m_AxisFrame);
	DDX_Check(pDX, IDC_INVISIBLE, m_Invisible);
	DDX_Check(pDX, IDC_WORLD_FRAME, m_WorldAxisFrame);
	//}}AFX_DATA_MAP
	DDX_Check(pDX, IDC_LIGHTING, m_Lighting);
	DDX_Check(pDX, IDC_GOURAUD_SHADED, m_GouraudShading);
	DDX_Check(pDX, IDC_WIRE_FRAME, m_WireFrame);
	DDX_Check(pDX, IDC_SHOW_NORMAL, m_ShowNormal);
	DDX_Check(pDX, IDC_SHOW_TEXTURE, m_ShowTexture);
	DDX_Check(pDX, IDC_TRANSPARENT, m_Transparent);
	DDX_Check(pDX, IDC_CONTROL_GRID, m_ControlGrid);
	DDX_Control(pDX, IDC_A_SLIDER, m_ASlider);
	DDX_Text(pDX, IDC_A_ECHO, m_ASliderEcho);
}


BEGIN_MESSAGE_MAP(TmodelControl, CDialog)
	//{{AFX_MSG_MAP(TmodelControl)
	ON_BN_CLICKED(IDC_CRAFT, OnCraft)
	ON_BN_CLICKED(IDC_ROTATE, OnRotate)
	ON_BN_CLICKED(IDC_SCALE, OnScale)
	ON_BN_CLICKED(IDC_TRANSLATE, OnTranslate)
	ON_WM_HSCROLL()
	//}}AFX_MSG_MAP
	ON_WM_CTLCOLOR()
	ON_BN_CLICKED(IDC_COLOR_EMISSIVE, OnBnClickedColorEmissive)
	ON_BN_CLICKED(IDC_COLOR_AMBIENT, OnBnClickedColorAmbient)
	ON_BN_CLICKED(IDC_COLOR_DIFFUSE, OnBnClickedColorDiffuse)
	ON_BN_CLICKED(IDC_COLOR_SPECULAR, OnBnClickedColorSpecular)
	ON_BN_CLICKED(IDC_POINT_LIGHT, OnBnClickedPointLight)
	ON_BN_CLICKED(IDC_DIRECTION_LIGHT, OnBnClickedDirectionLight)
	ON_BN_CLICKED(IDC_SPOT_LIGHT, OnBnClickedSpotLight)
	ON_BN_CLICKED(IDC_GROUND, OnBnClickedGround)
	ON_BN_CLICKED(IDC_TEAPOT, OnBnClickedTeapot)
END_MESSAGE_MAP()




#define kSliderScale	100.0f
#define kSliderFactor   (1.0f/kSliderScale)

void TmodelControl::UpdateSliderEcho(
	CString	&echo,
	float	value
)
{
	echo.Format("%5.2f", value);
	UpdateData(FALSE);
}


float TmodelControl::GetSliderBarValue(
	CSliderCtrl &slider
)
{
	return (slider.GetPos() * kSliderFactor);
}

void TmodelControl::InitializeSlider(
	CSliderCtrl &slider,
	CString	    &echo,
	float		min,
	float		max,
	float		init
)
{
	slider.SetRange(int(min*kSliderScale), int(max*kSliderScale), TRUE);
	slider.SetPos(int(init*kSliderScale));
	UpdateSliderEcho(echo, init);
}

void		
TmodelControl::SetSlidersToTranslate(float x, float y, float z)
{
	InitializeSlider(m_XSlider, m_XSliderEcho, -20, 20, x);
	InitializeSlider(m_YSlider, m_YSliderEcho, -20, 20, y);
	InitializeSlider(m_ZSlider, m_ZSliderEcho, -20, 20, z);
	m_ASlider.EnableWindow(FALSE);
}

void			
TmodelControl::SetSlidersToScale(float x, float y, float z)
{
	InitializeSlider(m_XSlider, m_XSliderEcho, 0.1, 20, x);
	InitializeSlider(m_YSlider, m_YSliderEcho, 0.1, 20, y);
	InitializeSlider(m_ZSlider, m_ZSliderEcho, 0.1, 20, z);
	m_ASlider.EnableWindow(FALSE);
}

void			
TmodelControl::SetSlidersToRotate(float x, float y, float z)
{
	InitializeSlider(m_XSlider, m_XSliderEcho, -180, 180, x);
	InitializeSlider(m_YSlider, m_YSliderEcho, -180, 180, y);
	InitializeSlider(m_ZSlider, m_ZSliderEcho, -180, 180, z);
	m_ASlider.EnableWindow(FALSE);
}

void TmodelControl::SetSlidersToColor(float r, float g, float b, float a)
{
	InitializeSlider(m_XSlider, m_XSliderEcho, 0, 1, r);
	InitializeSlider(m_YSlider, m_YSliderEcho, 0, 1, g);
	InitializeSlider(m_ZSlider, m_ZSliderEcho, 0, 1, b);
	m_ASlider.EnableWindow(TRUE);
	InitializeSlider(m_ASlider, m_ASliderEcho, 0, 1, a);
}


/////////////////////////////////////////////////////////////////////////////
// TmodelControl message handlers
BOOL TmodelControl::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	
	CheckRadioButton(IDC_SCALE, IDC_COLOR_SPECULAR,  IDC_COLOR_DIFFUSE);
	CheckRadioButton(IDC_CRAFT, IDC_GROUND,             IDC_GROUND);
	

	fXformType = eRotateXform;
	fObjectType = eGroundObject;
	SetSlidersToTranslate(0, 0, 0);
	
	m_AxisFrame = true;
	m_Invisible = false;

	m_WorldAxisFrame = false;

	UpdateData(FALSE);
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

BOOL TmodelControl::PreTranslateMessage(MSG* pMsg) 
{
	return (GetParent())->PreTranslateMessage(pMsg);
}


void TmodelControl::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	float v;
	v = GetSliderBarValue(m_XSlider);
	UpdateSliderEcho(m_XSliderEcho, v);

	v = GetSliderBarValue(m_YSlider);
	UpdateSliderEcho(m_YSliderEcho, v);

	v = GetSliderBarValue(m_ZSlider);
	UpdateSliderEcho(m_ZSliderEcho, v);

	v = GetSliderBarValue(m_ASlider);
	UpdateSliderEcho(m_ASliderEcho, v);
}

void TmodelControl::ControlSetObject()
{
	if (fCurrentObject == NULL)
		return;

	UpdateData(TRUE);	// get infor from the control

	if (m_AxisFrame) 
		fCurrentObject->EnableShowAxisFrame();
	else
		fCurrentObject->DisableShowAxisFrame();

	if (m_Invisible)
		fCurrentObject->DisableVisibility();
	else
		fCurrentObject->EnableVisibility();

	if (m_GouraudShading)
		fCurrentObject->SetShadedMode(D3DSHADE_GOURAUD);
	else
		fCurrentObject->SetShadedMode(D3DSHADE_FLAT);

	if (m_Lighting)
		fCurrentObject->EnableLighting();
	else 
		fCurrentObject->DisableLighting();
	
	if (m_WireFrame)
		fCurrentObject->SetFillMode(D3DFILL_WIREFRAME);
	else 
		fCurrentObject->SetFillMode(D3DFILL_SOLID);
	

	if (m_ShowNormal)
		fCurrentObject->EnableShowNormal();
	else
		fCurrentObject->DisableShowNormal();

	if (m_Transparent) 
		fCurrentObject->EnableAlphaBlending();
	else
		fCurrentObject->DisableAlphaBlending();

	if (m_ControlGrid)
		fCurrentObject->EnableShowControlGrid();
	else
		fCurrentObject->DisableShowControlGrid();

	if (m_ShowTexture)
		fCurrentObject->EnableShowTexture();
	else
		fCurrentObject->DisableShowTexture();


	float x = GetSliderBarValue(m_XSlider);
	float y = GetSliderBarValue(m_YSlider);
	float z = GetSliderBarValue(m_ZSlider);
	float a = GetSliderBarValue(m_ASlider);
	switch (fXformType) {
		case eTranslateXform:
			fCurrentObject->SetLocationTo(x, y, z);
			break;
		case eScaleXform:
			fCurrentObject->SetScale(x, y, z);
			break;
		case eRotateXform:
			fCurrentObject->GetXformInfo().UpdateRotation(x*M_PI/180.0, y*M_PI/180.0, z*M_PI/180.0);
			break;

			#define DoSetMaterialColor(_Type_)								\
					case eColor##_Type_##:									\
						{													\
							Tmaterial &mt = fCurrentObject->GetMaterial();	\
							mt.Set##_Type_##(x, y, z, a);						\
							break;											\
						}
			DoSetMaterialColor(Emissive)
			DoSetMaterialColor(Diffuse)
			DoSetMaterialColor(Ambient)
			DoSetMaterialColor(Specular)
	}
}

void TmodelControl::ObjectSetControl()
{
	if (fCurrentObject == NULL)
		return;

	m_AxisFrame = fCurrentObject->ShowAxisFrameIsEnabled();
	m_Invisible = !(fCurrentObject->VisibilityIsEnabled());
	m_GouraudShading = (fCurrentObject->GetShadedMode() == D3DSHADE_GOURAUD);
	m_Lighting = fCurrentObject->LightingIsEnabled();
	m_WireFrame = (fCurrentObject->GetFillMode() == D3DFILL_WIREFRAME);
	m_ShowNormal = 	fCurrentObject->ShowNormalIsEnabled();
	m_Transparent = fCurrentObject->AlphaBlendingIsEnabled();
	m_ControlGrid = fCurrentObject->ShowControlGridIsEnabled();
	m_ShowTexture = fCurrentObject->ShowTextureIsEnabled();
	
	switch (fXformType) {
		case eTranslateXform:
			{
				const D3DXVECTOR3 &v = fCurrentObject->GetLocation();
				SetSlidersToTranslate(v.x, v.y, v.z);
			}
			break;
		case eScaleXform:
			{
				const D3DXVECTOR3 &v = fCurrentObject->GetScale();
				SetSlidersToScale(v.x, v.y, v.z);
			}
			break;
		case eRotateXform:
			{
				const D3DXVECTOR3 &v = fCurrentObject->GetRotation();
				SetSlidersToRotate(v.x*180.0/M_PI, v.y*180.0/M_PI, v.z*180.0/M_PI);
			}
			break;

			#define DoSetSliderColor(_Type_)								\
				case eColor##_Type_##:										\
						{													\
							Tmaterial &mt = fCurrentObject->GetMaterial();	\
							D3DXCOLOR c;									\
							mt.Get##_Type_##(c);							\
							SetSlidersToColor(c.r, c.g, c.b, c.a);				\
							break;											\
						}
			DoSetSliderColor(Emissive)
			DoSetSliderColor(Diffuse)
			DoSetSliderColor(Ambient)
			DoSetSliderColor(Specular)
	}
	UpdateData(FALSE);	// get infor to the control
}

// I first saw the follow code in Vinh Weddle [CSS451 Winter 2003]'s mp0 solution
// Here is what Vinh said:
//		the following code is based on code that i found on msdn regarding
//		the OnCtlColor() member function
//
HBRUSH TmodelControl::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor) 
{
	HBRUSH hbr = CDialog::OnCtlColor(pDC, pWnd, nCtlColor);
	
	// TODO: Change any attributes of the DC here
	if (pWnd->GetDlgCtrlID() == IDC_X_LABEL)
	{
		pDC->SetTextColor( RGB(255,0,0) );
		pDC->SetBkMode(TRANSPARENT);
	}
	else if (pWnd->GetDlgCtrlID() == IDC_Y_LABEL)
	{
		pDC->SetTextColor( RGB(0,255,0) );
		pDC->SetBkMode(TRANSPARENT);
	}
	else if (pWnd->GetDlgCtrlID() == IDC_Z_LABEL)
	{
		pDC->SetTextColor( RGB(0,0,255) );
		pDC->SetBkMode(TRANSPARENT);
	}

	// TODO: Return a different brush if the default is not desired
	return hbr;
}


void TmodelControl::OnRotate() 
{
	fXformType = eRotateXform;
	ObjectSetControl();
}

void TmodelControl::OnScale() 
{
	fXformType = eScaleXform;
	ObjectSetControl();
}

void TmodelControl::OnTranslate() 
{
	fXformType = eTranslateXform;
	ObjectSetControl();
}

void TmodelControl::OnBnClickedColorEmissive()
{
	fXformType = eColorEmissive;
	ObjectSetControl();
}
void TmodelControl::OnBnClickedColorAmbient()
{
	fXformType = eColorAmbient;
	ObjectSetControl();
}

void TmodelControl::OnBnClickedColorDiffuse()
{
	fXformType = eColorDiffuse;
	ObjectSetControl();
}

void TmodelControl::OnBnClickedColorSpecular()
{
	fXformType = eColorSpecular;
	ObjectSetControl();
}

void TmodelControl::OnBnClickedPointLight()
{
	fCurrentObject= fWorld->GetPointLightObject();
	ObjectSetControl();
	fObjectType = ePointLightObject;
}

void TmodelControl::OnBnClickedDirectionLight()
{
	fCurrentObject = fWorld->GetDirectionalLightObject();
	ObjectSetControl();
	fObjectType = eDirectionalLightObject;
}

void TmodelControl::OnBnClickedSpotLight()
{
	fCurrentObject = fWorld->GetSpotLightObject();
	ObjectSetControl();
	fObjectType = eSpotLightObject;
}


void TmodelControl::OnCraft() 
{
	fCurrentObject = fWorld->GetCraftObject();
	ObjectSetControl();
	fObjectType = eCraftObject;
}


void TmodelControl::OnBnClickedGround()
{
	fCurrentObject = fWorld->GetGroundObject();
	ObjectSetControl();
	fObjectType = eGroundObject;
}
void TmodelControl::OnBnClickedTeapot()
{
	fCurrentObject = fWorld->GetTeapotObject();
	ObjectSetControl();
	fObjectType = eTeapotObject;
}
