// TrawMeshSurfaceControl.cpp : implementation file
//

#include "stdafx.h"
#include "ClassExample.h"
#include "TrawMeshSurfaceControl.h"

#include "./Collections/TmeshSurfaceLibrary.h"
#include "./Collections/TrawMeshSurface.h"

// TrawMeshSurfaceControl dialog

IMPLEMENT_DYNAMIC(TrawMeshSurfaceControl, CDialog)
TrawMeshSurfaceControl::TrawMeshSurfaceControl(CWnd* pParent /*=NULL*/)
	: CDialog(TrawMeshSurfaceControl::IDD, pParent)
	, m_ColumnEcho(_T(""))
	, m_ResolutionEcho(_T(""))
	, m_RowEcho(_T(""))
	, m_MeshXEcho(_T(""))
	, m_MeshYEcho(_T(""))
	, m_MeshZEcho(_T(""))
{
	fL1 = fL2 = NULL;
	fC = fR = 0;
}

TrawMeshSurfaceControl::~TrawMeshSurfaceControl()
{
}

void TrawMeshSurfaceControl::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Text(pDX, IDC_COLUMN_ECHO, m_ColumnEcho);
	DDX_Text(pDX, IDC_RESOLUTION_ECHO, m_ResolutionEcho);
	DDX_Text(pDX, IDC_ROW__ECHO, m_RowEcho);
	DDX_Control(pDX, IDC_COLUMN_SLIDER, m_ColumnSlider);
	DDX_Control(pDX, IDC_ROW_SLIDER, m_RowSlider);
	DDX_Control(pDX, IDC_RESOLUTION_SLIDER, m_ResolutionSlider);
	DDX_Text(pDX, IDC_MESH_X_ECHO, m_MeshXEcho);
	DDX_Text(pDX, IDC_MESH_Y_ECHO1, m_MeshYEcho);
	DDX_Text(pDX, IDC_MESH_Z_ECHO, m_MeshZEcho);
	DDX_Control(pDX, IDC_MESH_X_SLIDER, m_MeshXSlider);
	DDX_Control(pDX, IDC_MESH_Y_SLIDER, m_MeshYSlider);
	DDX_Control(pDX, IDC_MESH_Z_SLIDER, m_MeshZSlider);
}


BEGIN_MESSAGE_MAP(TrawMeshSurfaceControl, CDialog)
//	ON_WM_HSCROLL()
	ON_NOTIFY(NM_CUSTOMDRAW, IDC_MESH_X_SLIDER, OnNMCustomdrawMeshXSlider)
	ON_NOTIFY(NM_CUSTOMDRAW, IDC_MESH_Y_SLIDER, OnNMCustomdrawMeshYSlider)
	ON_NOTIFY(NM_CUSTOMDRAW, IDC_ROW_SLIDER, OnNMCustomdrawRowSlider)
	ON_NOTIFY(NM_CUSTOMDRAW, IDC_COLUMN_SLIDER, OnNMCustomdrawColumnSlider)
	ON_NOTIFY(NM_CUSTOMDRAW, IDC_MESH_Z_SLIDER, OnNMCustomdrawMeshZSlider)
	ON_NOTIFY(NM_CUSTOMDRAW, IDC_RESOLUTION_SLIDER, OnNMCustomdrawResolutionSlider)
END_MESSAGE_MAP()



#define kSliderFloatScale	10000.0f
#define kSliderFloatFactor   (1.0f/kSliderFloatScale)

#define kSliderIntScale    1
#define kSliderIntFactor   1

void TrawMeshSurfaceControl::UpdateSliderEcho(
	CString	&echo,
	float	value
)
{
	echo.Format("%5.2f", value);
	UpdateData(FALSE);
}


float TrawMeshSurfaceControl::GetSliderBarValue(
	CSliderCtrl &slider,
	float       fac
)
{
	return (slider.GetPos() * fac);
}

void TrawMeshSurfaceControl::InitializeSlider(
	CSliderCtrl &slider,
	CString	    &echo,
	float		min,
	float		max,
	float		init,
	float       scale
)
{
	slider.SetRange(int(min*scale), int(max*scale), TRUE);
	slider.SetPos(int(init*scale));
	UpdateSliderEcho(echo, init);
}


void TrawMeshSurfaceControl::SetMeshLibrary(
	TmeshSurfaceLibrary *l1, 
	TmeshSurfaceLibrary *l2) 
{ 
	fL1 = l1; 
	fL2 = l2; 
	fR = 0;
	fC = 0;
	
	frm1 = fL1->GetMeshSurfaceFromLibrary(0);
	frm1->SelectVertex(fR, fC);
	frm1->SelectedVertexPosition(fP);

	frm2 = fL2->GetMeshSurfaceFromLibrary(0);
	frm2->SelectVertex(fR, fC);
	frm2->SelectedVertexPosition(fP);

	SetControlVertexPos(fP.x, fP.y, fP.z);
}

void TrawMeshSurfaceControl::SetControlVertexPos(float x, float y, float z)
{
	InitializeSlider(m_MeshXSlider, m_MeshXEcho, x-0.05, x+0.05, x, kSliderFloatScale);
	InitializeSlider(m_MeshYSlider, m_MeshYEcho, y-0.20, y+0.20, y, kSliderFloatScale);
	InitializeSlider(m_MeshZSlider, m_MeshZEcho, z-0.05, z+0.05, z, kSliderFloatScale);
}

BOOL TrawMeshSurfaceControl::OnInitDialog()
{
	CDialog::OnInitDialog();

	SetControlVertexPos(0, 0, 0);

	InitializeSlider(m_ResolutionSlider, m_ResolutionEcho, 2, 50, 20, kSliderIntFactor);


	InitializeSlider(m_RowSlider, m_RowEcho, 0, 19, 0, kSliderIntFactor);
	InitializeSlider(m_ColumnSlider, m_ColumnEcho, 0, 19, 0, kSliderIntFactor);

	return TRUE;  // return TRUE unless you set the focus to a control
}

void TrawMeshSurfaceControl::OnNMCustomdrawMeshXSlider(NMHDR *pNMHDR, LRESULT *pResult)
{
	float x = GetSliderBarValue(m_MeshXSlider, kSliderFloatFactor);
	UpdateSliderEcho(m_MeshXEcho, x);
	fP.x = x;

	frm1->SetSelectedVertexPosition(fP);
	frm2->SetSelectedVertexPosition(fP);
}

void TrawMeshSurfaceControl::OnNMCustomdrawMeshYSlider(NMHDR *pNMHDR, LRESULT *pResult)
{
	float y = GetSliderBarValue(m_MeshYSlider, kSliderFloatFactor);
	UpdateSliderEcho(m_MeshYEcho, y);
	fP.y = y;

	frm1->SetSelectedVertexPosition(fP);
	frm2->SetSelectedVertexPosition(fP);
}


void TrawMeshSurfaceControl::OnNMCustomdrawMeshZSlider(NMHDR *pNMHDR, LRESULT *pResult)
{
	float z = GetSliderBarValue(m_MeshZSlider, kSliderFloatFactor);
	UpdateSliderEcho(m_MeshZEcho, z);
	fP.z = z;

	frm1->SetSelectedVertexPosition(fP);
	frm2->SetSelectedVertexPosition(fP);
}

void TrawMeshSurfaceControl::OnNMCustomdrawRowSlider(NMHDR *pNMHDR, LRESULT *pResult)
{
	fR = GetSliderBarValue(m_RowSlider, kSliderIntScale);
	UpdateSliderEcho(m_RowEcho, fR);

	frm1->SelectVertex(fR, fC);
	frm2->SelectVertex(fR, fC);
	frm1->SelectedVertexPosition(fP);
	
	SetControlVertexPos(fP.x, fP.y, fP.z);
}

void TrawMeshSurfaceControl::OnNMCustomdrawColumnSlider(NMHDR *pNMHDR, LRESULT *pResult)
{
	fC = GetSliderBarValue(m_ColumnSlider, kSliderIntScale);
	UpdateSliderEcho(m_ColumnEcho, fC);
	
	frm1->SelectVertex(fR, fC);
	frm2->SelectVertex(fR, fC);
	frm1->SelectedVertexPosition(fP);

	SetControlVertexPos(fP.x, fP.y, fP.z);
}

void TrawMeshSurfaceControl::OnNMCustomdrawResolutionSlider(NMHDR *pNMHDR, LRESULT *pResult)
{
	int res = GetSliderBarValue(m_ResolutionSlider, kSliderIntScale);
	UpdateSliderEcho(m_ResolutionEcho, res);

	if (frm1->Resolution() != res) {
		fL1->CreateMeshSurface(0, res);
		fL2->CreateMeshSurface(0, res);

		frm1 = fL1->GetMeshSurfaceFromLibrary(0);
		frm2 = fL2->GetMeshSurfaceFromLibrary(0);
		
		InitializeSlider(m_RowSlider, m_RowEcho, 0, res, 0, kSliderIntFactor);
		InitializeSlider(m_ColumnSlider, m_ColumnEcho, 0, res, 0, kSliderIntFactor);
		fC = fR = 0;
	}
}
